# ----------------------------------------------------------------------------
# pyglet
# Copyright (c) 2006-2008 Alex Holkner
# Copyright (c) 2008-2019 pyglet contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of pyglet nor the names of its
#    contributors may be used to endorse or promote products
#    derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
"""
Responsabilities

    Handles accumulation of debug events while playing media_player and  saves
    when sample's play ends.
"""
from __future__ import division, print_function, unicode_literals

import pickle
import time


class BufferedLogger(object):
    def __init__(self, outfile):
        self.outfile = outfile
        self.log_entries = []
        self.start_wall_time = None
        # (fn, args)
        self.on_close_callback_info = None

    def init_wall_time(self):
        self.start_wall_time = time.perf_counter()

    def log(self, *args):
        self.log_entries.append(args)

    def rebased_wall_time(self):
        return time.perf_counter() - self.start_wall_time

    def close(self):
        self.save_log_entries_as_pickle()
        if self.on_close_callback_info is not None:
            fn, args = self.on_close_callback_info
            fn(self.log_entries, *args)

    def save_log_entries_as_pickle(self):
        with open(self.outfile, "wb") as f:
            pickle.dump(self.log_entries, f)

    def clear(self):
        self.log_entries = []


logger = None
